#!/usr/bin/env python
#
# Copyright (C): 2008 Per B. Sederberg
# Authors: Per B. Sederberg
# URL: http://memory.psych.upenn.edu/publications
#
# Distributed under the terms of the GNU General Public License
# (GPL). See the license.txt that came with this file.
#

#
# Simulate TCM-A
#

# load necessary modules
from tcma import TCMA
import numpy as np

try:
    import pylab as pl
    doPlots = True
except:
    print "MatPlotLib is not installed, so no plots will be generated."
    doPlots = False
    
# set up the parameters for the simulation
listLength = 12
numLists = 1000
params = {'pScale': 1.655383,
          'pDecay': 1.981120091,
          'B_enc': 0.62676308,
          'B_ret': 0.62676308,
          'B_dist': 0.00, 
          'eta': 0.29934,
          'L': .62,
          'K': .62,
          'minI': .0001,
          'Gft_ret': .4454162578,
          'Gtf': 0.80,
          'Gin': 1.0,
          'recDuration': 60000,
          'thresh': 1.0,
          'dt': 100,
          'tau': 200
          }


# SPC function
def calcSPC(recalls,listLength):
    """
    Calculate an SPC from the recalls.
    """
    numLists = recalls.shape[0]
    prec = np.zeros(listLength)
    for i in range(1,listLength+1):
        prec[i-1] = ((recalls == i).sum(1) > 0).sum()/float(numLists)
    return prec

# simulate IFR
print "Simulating IFR..."

# make the model instance
tcma = TCMA(params)
isContDistract = False
recalls,recTimes,inputs,CVs = tcma.Simulate(numLists,
                                            listLength,
                                            isContDistract)
prec_IFR = calcSPC(recalls, listLength)


# simulate DFR
print "Simulating DFR..."

# set the distractor drift rate
params['B_dist'] = 0.9760686

# make the model instance
tcma = TCMA(params)
isContDistract = False
recalls,recTimes,inputs,CVs = tcma.Simulate(numLists,
                                            listLength,
                                            isContDistract)
prec_DFR = calcSPC(recalls, listLength)


# simulate CDFR
print "Simulating CDFR..."

# set the distractor drift rate
params['B_dist'] = 0.9760686

# make the model instance
tcma = TCMA(params)
isContDistract = True
recalls,recTimes,inputs,CVs = tcma.Simulate(numLists,
                                            listLength,
                                            isContDistract)
prec_CDFR = calcSPC(recalls, listLength)

# tell the percent recalled
print "Percent recalled in each condition:"
print "IFR:\t%g" % (prec_IFR.mean())
print "DFR:\t%g" % (prec_DFR.mean())
print "CDFR:\t%g" % (prec_CDFR.mean())

# do the plots
if doPlots:
    pl.figure(1)
    pl.clf()
    xvals = range(1,listLength+1)
    pl.plot(xvals,prec_IFR,'k-')
    pl.plot(xvals,prec_DFR,'k--')
    pl.plot(xvals,prec_CDFR,'k:')
    pl.xlim([0,listLength+1])
    pl.ylim([0,1])
    pl.xlabel('Serial Pos.')
    pl.ylabel('P(Recall)')
    pl.legend(('IFR','DFR','CDFR'),loc=0)
    pl.show()
    
