#include <stdlib.h>
#include <math.h>
#include "helper.h"

long accumulate(double* matrix, int rows, int cols, 
		double K, double L, double eta, double thresh,
		double dt_tau, double sq_dt_tau) {
  int i, index;
  long t;

  int crossed = 0;

  int numItems = cols;
  long maxTime = rows-3;

  double lx[numItems];  // lateral inhibition
  double sumx;

  // get x and randn from matrix
  double *x = &matrix[0];
  double *I = &matrix[numItems];
  double *recalled = &matrix[2*numItems];
  double *randn_vals = &matrix[3*numItems];

  // loop over time
  for (t=0; t<maxTime; t++)
  {
    // get the sum for the lateral inhibition
    sumx = 0;
    for (i=0; i<numItems; i++)
    {
	sumx += x[i];
    }

    // loop over items
    for (i=0; i<numItems; i++)
    {
      // calculate the lateral inhibition
      lx[i] = sumx - x[i];

      // get index into randn
      index = t*numItems + i;

      // determine the change in x
      x[i] += (I[i] - K*x[i] - L*lx[i])*dt_tau + (eta*randn_vals[index]*sq_dt_tau);

      // make sure not below zero
      if (x[i] < 0)
      {
	x[i] = 0;
      }
      
      // see if crossed thresh
      if (x[i] >= thresh)
      {
	// crossed, now see if it is already recalled
	if (recalled[i] == 0)
	{
	  crossed = 1;
	}
	else
	{
	  // already recalled, so bump down to zero
	  x[i] = 0;
	}
      }
    }

    if (crossed == 1)
    {
      break;
    }

  }

  // we are done
  return t;

}



